@@ -121,10 +121,10 @@ func (fs *FileServer) earlyBreakParameters(w http.ResponseWriter, req *http.Requ
 	}
 	if _, ok := req.URL.Query()["embedded"]; ok {
 		if err := fs.embedded(w, req); err != nil {
-			logger.LogRequest(req, http.StatusNotFound, fs.Verbose)
+			logger.LogRequest(req, http.StatusNotFound, fs.Verbose, fs.Webhook)
 			return true
 		}
-		logger.LogRequest(req, http.StatusOK, fs.Verbose)
+		logger.LogRequest(req, http.StatusOK, fs.Verbose, fs.Webhook)
 		return true
 	}
 	if _, ok := req.URL.Query()["delete"]; ok {
@@ -188,7 +188,7 @@ func (fs *FileServer) handler(w http.ResponseWriter, req *http.Request) {
 	defer file.Close()
 
 	// Log request
-	logger.LogRequest(req, http.StatusOK, fs.Verbose)
+	logger.LogRequest(req, http.StatusOK, fs.Verbose, fs.Webhook)
 
 	// Switch and check if dir
 	stat, _ := file.Stat()
@@ -239,6 +239,7 @@ func (fileS *FileServer) constructEmbedded() []item {
 				item.Ext = strings.ToLower(utils.ReturnExt(d.Name()))
 				uri_temp := url.PathEscape(pathS)
 				uri_temp = strings.TrimPrefix(uri_temp, "embedded")
+				uri_temp = strings.TrimPrefix(uri_temp, "%2F")
 				item.URI = fmt.Sprintf("%s?embedded", uri_temp)
 
 				// Add to items slice
@@ -500,7 +501,7 @@ func (fs *FileServer) sendFile(w http.ResponseWriter, req *http.Request, file *o
 		}
 
 		// Send webhook message
-		fs.HandleWebhookSend(fmt.Sprintf("File downloaded: %s", filepath.Join(fs.Webroot, req.URL.Path)), "download")
+		logger.HandleWebhookSend(fmt.Sprintf("[WEB] File downloaded: %s", filepath.Join(fs.Webroot, req.URL.Path)), "download", fs.Webhook)
 
 	} else {
 		// Write to browser
@@ -514,7 +515,7 @@ func (fs *FileServer) sendFile(w http.ResponseWriter, req *http.Request, file *o
 		}
 
 		// Send webhook message
-		fs.HandleWebhookSend(fmt.Sprintf("File viewed: %s", filepath.Join(fs.Webroot, req.URL.Path)), "view")
+		logger.HandleWebhookSend(fmt.Sprintf("[WEB] File viewed: %s", filepath.Join(fs.Webroot, req.URL.Path)), "view", fs.Webhook)
 	}
 }
 
@@ -547,7 +548,7 @@ func (fs *FileServer) deleteFile(w http.ResponseWriter, req *http.Request) {
 	}
 
 	// Send webhook message
-	fs.HandleWebhookSend(fmt.Sprintf("File deleted: %s", deletePath), "delete")
+	logger.HandleWebhookSend(fmt.Sprintf("[WEB] File deleted: %s", deletePath), "delete", fs.Webhook)
 
-	logger.LogRequest(req, http.StatusResetContent, fs.Verbose)
+	logger.LogRequest(req, http.StatusResetContent, fs.Verbose, fs.Webhook)
 }
