@@ -10,16 +10,18 @@ import (
 	"strings"
 	"syscall"
 
+	"github.com/howeyc/gopass"
 	"github.com/patrickhener/goshs/ca"
 	"github.com/patrickhener/goshs/config"
+	"github.com/patrickhener/goshs/goshsversion"
 	"github.com/patrickhener/goshs/httpserver"
 	"github.com/patrickhener/goshs/logger"
+	"github.com/patrickhener/goshs/sftpserver"
 	"github.com/patrickhener/goshs/update"
 	"github.com/patrickhener/goshs/utils"
+	"github.com/patrickhener/goshs/webhook"
 )
 
-const goshsVersion = "v1.0.6"
-
 var (
 	port                = 8000
 	ip                  = "0.0.0.0"
@@ -31,10 +33,15 @@ var (
 	myKey               = ""
 	myCert              = ""
 	myP12               = ""
+	p12NoPass           = false
 	basicAuth           = ""
 	certAuth            = ""
 	webdav              = false
 	webdavPort          = 8001
+	sftp                = false
+	sftpPort            = 2022
+	sftpKeyfile         = ""
+	sftpHostKeyfile     = ""
 	uploadOnly          = false
 	readOnly            = false
 	noClipboard         = false
@@ -49,7 +56,6 @@ var (
 	embedded            = false
 	output              = ""
 	configFile          = ""
-	printConfig         = false
 	webhookEnable       = false
 	webhookURL          = ""
 	webhookEvents       = "all"
@@ -80,16 +86,23 @@ Web server options:
   -o,  --output       Write output to logfile                 (default: false)
 
 TLS options:
-  -s,   --ssl          Use TLS
-  -ss,  --self-signed  Use a self-signed certificate
-  -sk,  --server-key   Path to server key
-  -sc,  --server-cert  Path to server certificate
-  -p12, --pkcs12       Path to server p12
-  -sl,  --lets-encrypt Use Let's Encrypt as certification service
-  -sld, --le-domains   Domain(s) to request from Let's Encrypt		(comma separated list)
-  -sle, --le-email     Email to use with Let's Encrypt
-  -slh, --le-http      Port to use for Let's Encrypt HTTP Challenge	(default: 80)
-  -slt, --le-tls       Port to use for Let's Encrypt TLS ALPN Challenge (default: 443)
+  -s,     --ssl           Use TLS
+  -ss,    --self-signed   Use a self-signed certificate
+  -sk,    --server-key    Path to server key
+  -sc,    --server-cert   Path to server certificate
+  -p12,   --pkcs12        Path to server p12
+  -p12np, --p12-no-pass   Server p12 has empty password
+  -sl,    --lets-encrypt  Use Let's Encrypt as certification service
+  -sld,   --le-domains    Domain(s) to request from Let's Encrypt		(comma separated list)
+  -sle,   --le-email      Email to use with Let's Encrypt
+  -slh,   --le-http       Port to use for Let's Encrypt HTTP Challenge	(default: 80)
+  -slt,   --le-tls        Port to use for Let's Encrypt TLS ALPN Challenge (default: 443)
+
+SFTP server options:
+  -sftp                        Activate SFTP server capabilities (default: false)
+  -sp,    --sftp-port          The port SFTP listens on          (default: 2022)
+  -skf,   --sftp-keyfile       Authorized_keys file for pubkey auth
+  -shk,   --sftp-host-keyfile  SSH Host key file for identification
 
 Authentication options:
   -b,  --basic-auth    Use basic authentication (user:pass - user can be empty)
@@ -97,12 +110,12 @@ Authentication options:
   -H,  --hash          Hash a password for file based ACLs
 
 Webhook options:
-  -W,  --webhook            Enable webhook support                                      (default: false)
+  -W,  --webhook            Enable webhook support                                       (default: false)
   -Wu, --webhook-url        URL to send webhook requests to
   -We, --webhook-events     Comma separated list of events to notify
-                            [all, upload, delete, download, view, verbose]              (default: all)
+                            [all, upload, delete, download, view, webdav, sftp, verbose] (default: all)
   -Wp, --webhook-provider   Webhook provider
-                            [Discord, Mattermost, Slack]                                (default: Discord)
+                            [Discord, Mattermost, Slack]                                 (default: Discord)
 
 Misc options:
   -C  --config        Provide config file path                (default: false)
@@ -125,7 +138,7 @@ Usage examples:
   Start with basic auth empty user:	./goshs -b ':$up3r$3cur3'
   Start with cli enabled:           	./goshs -b 'secret-user:$up3r$3cur3' -s -ss -c
 
-`, goshsVersion, os.Args[0])
+`, goshsversion.GoshsVersion, os.Args[0])
 	}
 }
 
@@ -150,6 +163,8 @@ func flags() (*bool, *bool, *bool, *bool, *bool, *bool) {
 	flag.StringVar(&myCert, "server-cert", myCert, "server cert")
 	flag.StringVar(&myP12, "p12", myP12, "server p12")
 	flag.StringVar(&myP12, "pkcs12", myP12, "server p12")
+	flag.BoolVar(&p12NoPass, "p12np", p12NoPass, "p12 no pass")
+	flag.BoolVar(&p12NoPass, "p12-no-pass", p12NoPass, "p12 no pass")
 	flag.StringVar(&basicAuth, "b", basicAuth, "basic auth")
 	flag.StringVar(&basicAuth, "basic-auth", basicAuth, "basic auth")
 	flag.StringVar(&certAuth, "ca", certAuth, "cert auth")
@@ -188,7 +203,7 @@ func flags() (*bool, *bool, *bool, *bool, *bool, *bool) {
 	flag.BoolVar(&embedded, "embedded", embedded, "")
 	flag.StringVar(&output, "o", output, "")
 	flag.StringVar(&output, "output", output, "")
-	flag.BoolVar(&webhookEnable, "Wh", webhookEnable, "")
+	flag.BoolVar(&webhookEnable, "W", webhookEnable, "")
 	flag.BoolVar(&webhookEnable, "webhook", webhookEnable, "")
 	flag.StringVar(&webhookURL, "Wu", webhookURL, "")
 	flag.StringVar(&webhookURL, "webhook-url", webhookURL, "")
@@ -196,6 +211,14 @@ func flags() (*bool, *bool, *bool, *bool, *bool, *bool) {
 	flag.StringVar(&webhookEvents, "webhook-events", webhookEvents, "")
 	flag.StringVar(&webhookProvider, "Wp", webhookProvider, "")
 	flag.StringVar(&webhookProvider, "webhook-provider", webhookProvider, "")
+	flag.BoolVar(&sftp, "sftp", sftp, "sftp")
+	flag.IntVar(&sftpPort, "sp", sftpPort, "sftp port")
+	flag.IntVar(&sftpPort, "sftp-port", sftpPort, "sftp port")
+	flag.StringVar(&sftpKeyfile, "skf", sftpKeyfile, "")
+	flag.StringVar(&sftpKeyfile, "sftp-keyfile", sftpKeyfile, "")
+	flag.StringVar(&sftpHostKeyfile, "shk", sftpHostKeyfile, "")
+	flag.StringVar(&sftpHostKeyfile, "sftp-host-keyfile", sftpHostKeyfile, "")
+
 	updateGoshs := flag.Bool("update", false, "update")
 	hash := flag.Bool("H", false, "hash")
 	hashLong := flag.Bool("hash", false, "hash")
@@ -247,6 +270,11 @@ func sanityChecks() {
 		logger.Fatal("To use certificate based authentication with a CA cert you will need tls in any mode (-ss, -sk/-sc, -p12, -sl)")
 	}
 
+	// Sanity check either user:pass or keyfile when using sftp
+	if sftp && (basicAuth == "" && sftpKeyfile == "") {
+		logger.Fatal("When using SFTP you need to either specify an authorized keyfile using -sfk or username and password using -b")
+	}
+
 }
 
 // Flag handling
@@ -264,24 +292,33 @@ func init() {
 	}
 
 	if *updateGoshs {
-		err := update.UpdateTool(goshsVersion)
+		err := update.UpdateTool(goshsversion.GoshsVersion)
 		if err != nil {
 			logger.Fatalf("Failed to update tool: %+v", err)
 		}
 	}
 
 	if *version {
-		fmt.Printf("goshs version is: %+v\n", goshsVersion)
+		fmt.Printf("goshs version is: %+v\n", goshsversion.GoshsVersion)
 		os.Exit(0)
 	}
 
 	if *hash || *hashLong {
-		utils.GenerateHashedPassword()
+		fmt.Printf("Enter password: ")
+		password, err := gopass.GetPasswdMasked()
+		if err != nil {
+			logger.Fatalf("error reading password from stdin: %+v", err)
+		}
+		utils.GenerateHashedPassword(password)
 		os.Exit(0)
 	}
 
 	if *printConfig || *printConfigLong {
-		config.PrintExample()
+		config, err := config.PrintExample()
+		if err != nil {
+			panic(err)
+		}
+		fmt.Println(config)
 		os.Exit(0)
 	}
 
@@ -306,6 +343,7 @@ func init() {
 		myKey = cfg.PrivateKey
 		myCert = cfg.Certificate
 		myP12 = cfg.P12
+		p12NoPass = cfg.P12NoPass
 		letsencrypt = cfg.LetsEncrypt
 		leDomains = cfg.LetsEncryptDomain
 		leEmail = cfg.LetsEncryptEmail
@@ -329,6 +367,10 @@ func init() {
 		webhookURL = cfg.WebhookURL
 		webhookProvider = cfg.WebhookProvider
 		webhookEventsParsed = cfg.WebhookEvents
+		sftp = cfg.SFTP
+		sftpPort = cfg.SFTPPort
+		sftpKeyfile = cfg.SFTPKeyFile
+		sftpHostKeyfile = cfg.SFTPHostKeyFile
 
 		// Abspath for webroot
 		// Trim trailing / for linux/mac and \ for windows
@@ -354,9 +396,10 @@ func init() {
 	sanityChecks()
 
 	if webdav {
-		logger.Warn("upload/read-only mode deactivated due to use of 'webdav' mode")
+		logger.Warn("upload/read-only/no-delete mode deactivated due to use of 'webdav' mode")
 		uploadOnly = false
 		readOnly = false
+		noDelete = false
 	}
 
 	// Abspath for webroot
@@ -385,13 +428,13 @@ func parseBasicAuth() (string, string) {
 }
 
 func main() {
-	if yes, out := update.CheckForUpdates(goshsVersion); yes {
+	if yes, out := update.CheckForUpdates(goshsversion.GoshsVersion); yes {
 		logger.Warnf("There is a newer Version (%s) of goshs available. Run --update to update goshs.", out)
 	} else {
 		if out != "" {
 			logger.Warnf("Failed to check for updates: %+v", out)
 		} else {
-			logger.Infof("You are running the newest version (%s) of goshs", goshsVersion)
+			logger.Infof("You are running the newest version (%s) of goshs", goshsversion.GoshsVersion)
 		}
 	}
 
@@ -430,44 +473,71 @@ func main() {
 		logger.LogFile(multiWriter)
 	}
 
+	// Register webhook
+	webhook := webhook.Register(webhookEnable, webhookURL, webhookProvider, webhookEventsParsed)
+
 	// Setup the custom file server
 	server := &httpserver.FileServer{
-		IP:              ip,
-		Port:            port,
-		CLI:             cli,
-		Webroot:         webroot,
-		SSL:             ssl,
-		SelfSigned:      selfsigned,
-		LetsEncrypt:     letsencrypt,
-		MyCert:          myCert,
-		MyKey:           myKey,
-		MyP12:           myP12,
-		User:            user,
-		Pass:            pass,
-		CACert:          certAuth,
-		DropUser:        dropuser,
-		UploadOnly:      uploadOnly,
-		ReadOnly:        readOnly,
-		NoClipboard:     noClipboard,
-		NoDelete:        noDelete,
-		Silent:          silent,
-		Embedded:        embedded,
-		WebhookEnable:   webhookEnable,
-		WebhookURL:      webhookURL,
-		WebhookEvents:   webhookEventsParsed,
-		WebhookProvider: webhookProvider,
-		Verbose:         verbose,
-		Version:         goshsVersion,
+		IP:          ip,
+		Port:        port,
+		CLI:         cli,
+		Webroot:     webroot,
+		SSL:         ssl,
+		SelfSigned:  selfsigned,
+		LetsEncrypt: letsencrypt,
+		MyCert:      myCert,
+		MyKey:       myKey,
+		MyP12:       myP12,
+		P12NoPass:   p12NoPass,
+		User:        user,
+		Pass:        pass,
+		CACert:      certAuth,
+		DropUser:    dropuser,
+		UploadOnly:  uploadOnly,
+		ReadOnly:    readOnly,
+		NoClipboard: noClipboard,
+		NoDelete:    noDelete,
+		Silent:      silent,
+		Embedded:    embedded,
+		Webhook:     *webhook,
+		Verbose:     verbose,
+		Version:     goshsversion.GoshsVersion,
 	}
 
+	// Zeroconf mDNS
+	err := utils.RegisterZeroconfMDNS(ssl, port, webdav, webdavPort, sftp, sftpPort)
+	if err != nil {
+		logger.Warnf("error registering zeroconf mDNS: %+v", err)
+	}
+
+	// Start web server
 	go server.Start("web")
 
+	// Start WebDAV server
 	if webdav {
 		server.WebdavPort = webdavPort
 
 		go server.Start("webdav")
 	}
 
+	// Start SFTP server
+	if sftp {
+		s := &sftpserver.SFTPServer{
+			IP:          ip,
+			Port:        sftpPort,
+			KeyFile:     sftpKeyfile,
+			Username:    user,
+			Password:    pass,
+			Root:        webroot,
+			ReadOnly:    readOnly,
+			UploadOnly:  uploadOnly,
+			HostKeyFile: sftpHostKeyfile,
+			Webhook:     *webhook,
+		}
+
+		go s.Start()
+	}
+
 	<-done
 
 	logger.Infof("Received CTRL+C, exiting...")
